import * as d3 from 'd3';
import { feature } from 'topojson';

var projections = [
  { name: 'mercator', value: d3.geoMercator() },
  { name: 'orthograpic', value: d3.geoOrthographic() },
  { name: 'Us', value: d3.geoAlbersUsa() },
];

let projection = projections[1].value;
const svg = d3.select('svg');
const graticule = d3.geoGraticule();
let pathGenerator = d3.geoPath(projection);
let country;

const g = svg.append('g');

g.append('path')
  .attr('class', 'sphere')
  .attr('d', pathGenerator({ type: 'Sphere' }));

g.append('path')
  .datum(graticule)
  .attr('class', 'graticule')
  .attr('d', pathGenerator);

g.call(
  d3.drag().on('drag', (event, d) => {
    //if (projection === projections[1].value){
    const rotate = projection.rotate();
    const k = 75 / projection.scale();
    projection.rotate([
      rotate[0] + event.dx * k,
      rotate[1] - event.dy * k,
    ]);
    pathGenerator = d3.geoPath().projection(projection);
    svg.selectAll('.graticule').attr('d', pathGenerator);
    svg.selectAll('.country').attr('d', pathGenerator);
    //}
    //else{return}
  })
);

function update() {
  pathGenerator = d3.geoPath().projection(projection);
  svg.selectAll('path').attr('d', pathGenerator);
  svg.selectAll('path').transition().duration(750);
  svg
    .selectAll('.sphere')
    .attr('d', pathGenerator({ type: 'Sphere' }));
}

Promise.all([
  d3.tsv(
    'https://unpkg.com/world-atlas@1.1.4/world/110m.tsv'
  ),
  d3.json(
    'https://cdn.jsdelivr.net/npm/world-atlas@2/countries-110m.json'
  )
]).then(([tsvData, topoJSONdata]) => {
  const countryName = tsvData.reduce((accumulator, d) => {
    //for each..
    accumulator[d.iso_n3] = d.name;
    return accumulator;
  });

  const countries = feature(
    topoJSONdata,
    topoJSONdata.objects.countries
  );
  console.log(projection);
  g.selectAll('.country')
    .data(countries.features)
    .enter()
    .append('path')
    .attr('class', 'country')
    .attr('d', pathGenerator)
    .on('click', function (event, d) {
      country = countryName[d.id];
      if (
        country === 'United States' &&
        projection !== projections[0].value
      ) {
        console.log(projection);
        projection = projections[0].value;
        update();
      } else {
        projection = projections[1].value;
        update();
      }
    });
});

g.call(
  d3.zoom().on('zoom', (event) => {
    g.attr('transform', event.transform);
  })
);
